@@ -9,7 +9,7 @@ require Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT_OK = qw(wrap_sub);
 
-our $VERSION = '0.13'; # VERSION
+our $VERSION = '0.14'; # VERSION
 
 our %SPEC;
 
@@ -153,7 +153,63 @@ sub _code_as_str {
     join "\n", @lines;
 }
 
-sub handlemeta_v { {} }
+sub handlemeta_v { {prio=>0.1, convert=>1} }
+sub handle_v {
+    my ($self, %args) = @_;
+
+    my $v      = $args{new} // $args{value};
+    die "Cannot produce metadata other than v1.1 ($v)" unless $v == 1.1;
+
+    return if $v == $args{value};
+    die "Cannot convert metadata other than from v1.0"
+        unless $args{value} == 1.0;
+
+    my $meta = $self->{_meta};
+
+    # converting metadata from v1.0 to v1.1
+    if ($meta->{args}) {
+        for my $a (keys %{$meta->{args}}) {
+            my $old = $meta->{args}{$a};
+            my $new = {};
+            if (ref($old) eq 'ARRAY') {
+                if (defined $old->[1]{arg_pos}) {
+                    $new->{pos} = $old->[1]{arg_pos};
+                    delete $old->[1]{arg_pos};
+                }
+                if (defined $old->[1]{arg_greedy}) {
+                    $new->{greedy} = $old->[1]{arg_greedy};
+                    delete $old->[1]{arg_greedy};
+                }
+                if (defined $old->[1]{arg_complete}) {
+                    $new->{completion} = $old->[1]{arg_complete};
+                    delete $old->[1]{arg_complete};
+                }
+                if (defined $old->[1]{arg_aliases}) {
+                    # i'm lazy
+                    warn "Can't handle arg_aliases yet ".
+                        "(arg '$a' in property 'args'), ignored. You can move ".
+                            "this manually to a new arg with 'alias_for' set ".
+                                "to the canonical arg.";
+                    delete $old->[1]{arg_aliases};
+                }
+            } elsif (!ref($old)) {
+                # do nothing
+            } else {
+                die "Can't handle v1.0 args property (not array/scalar)";
+            }
+            $new->{schema} = $old;
+            $meta->{args}{$a} = $new;
+        }
+    }
+
+    if ($meta->{result}) {
+        $meta->{result} = {schema=>$meta->{result}};
+    }
+
+    $meta->{_note} = "Converted from v1.0 by ".__PACKAGE__.
+        " on ".scalar(localtime);
+}
+
 sub handlemeta_default_lang { {} }
 sub handlemeta_name { {} }
 sub handlemeta_summary { {} }
@@ -366,11 +422,14 @@ sub wrap {
         $meta->{$_} = undef unless exists $meta->{$_};
     }
 
+    $convert->{v} //= 1.1;
+
     # clone some properties
 
-    my $v = $meta->{v} // 1.0;
-    return [412, "Unsupported metadata version ($v), only 1.1 supported"]
-        unless $v == 1.1;
+    $meta->{v} //= 1.0;
+    return [412, "Unsupported metadata version ($meta->{v}), only 1.0 & 1.1 ".
+                "supported"]
+        unless $meta->{v} == 1.1 || $meta->{v} == 1.0;
 
     # put the sub in a named variable, so it can be accessed by the wrapper
     # code.
@@ -380,7 +439,7 @@ sub wrap {
     # also store the meta, it is needed by the wrapped sub. sometimes the meta
     # contains coderef and can't be dumped reliably, so we store it instead.
     my $metaname = $comppkg . "::meta".Scalar::Util::refaddr($meta);
-    { no strict 'refs'; ${$metaname} = $meta; }
+    { no strict 'refs'; no warnings; ${$metaname} = $meta; }
     $self->{_var_meta} = $metaname;
 
     # reset work variables. we haven't tested this yet because we expect the
@@ -535,7 +594,7 @@ So far you can convert 'args_as' and 'result_naked'.
 _
         },
         trap => {
-            schema => 'bool',
+            schema => ['bool' => {default=>1}],
             summary => 'Whether to trap exception using an eval block',
             description => <<'_',
 
@@ -544,10 +603,9 @@ function dies. Note that if some other properties requires an eval block (like
 'timeout') an eval block will be added regardless of this parameter.
 
 _
-            default => 1,
         },
         compile => {
-            schema => 'bool',
+            schema => ['bool' => {default=>1}],
             summary => 'Whether to compile the generated wrapper',
             description => <<'_',
 
@@ -555,7 +613,17 @@ Can be set to 0 to not actually wrap but just return the generated wrapper
 source code.
 
 _
-            default => 1,
+        },
+        normalize_schema => {
+            schema => ['bool' => {default=>1}],
+            summary => 'Whether to normalize schemas in metadata',
+            description => <<'_',
+
+By default, wrapper normalize Sah schemas in metadata, like in 'args' or
+'result' property, for convenience so that it does not need to be normalized
+again prior to use. If you want to turn off this behaviour, set to false.
+
+_
         },
     },
 };
@@ -576,7 +644,7 @@ Perinci::Sub::Wrapper - A multi-purpose subroutine wrapping framework
 
 =head1 VERSION
 
-version 0.13
+version 0.14
 
 =head1 SYNOPSIS
 
